@@ -261,6 +261,7 @@ def boxplot01(df, parameter_name, title):
     std_error = round(np.std(par))
     plt.axhline(y=mean, ls='--', alpha=0.5, c='blue', lw=1)
     plt.text(1.1, mean, '→ (mean:' + str(mean) + ' | std-err:' + str(std_error) + ')', size=9, c='blue')
+    ax1.set_xticklabels(labels,rotation=45,fontsize=9)
 
     plt.show()
 
@@ -636,7 +637,7 @@ def bar01(df, parameter_x, title):  # single category
     plt.bar(x_pos, y, color='steelblue', edgecolor='black', lw=1, alpha=0.9)
     plt.xlabel(parameter_x)
     plt.ylabel('Counts')
-    plt.xticks(x_pos, label, fontproperties=font)
+    plt.xticks(x_pos, label, rotation=45, fontproperties=font)
     plt.title(title)
     plt.show()
 
@@ -730,7 +731,7 @@ def bar02(df, parameter_x, parameter_category, title):  # two categories. y=coun
 
     plt.grid(linestyle='--', lw=0.5)
 
-    plt.xticks(ind + (n - 1) * width / 2, (labelx), fontproperties=font)
+    plt.xticks(ind + (n - 1) * width / 2, (labelx), rotation=45, fontproperties=font)
     plt.xlabel(parameter_x)
     plt.ylabel(parameter_category)
     plt.title(title)
@@ -786,7 +787,7 @@ def bar03(df, parameter_x, parameter_y, method, title):  # single category
     plt.xlabel(parameter_x)
     plt.ylabel(parameter_y + ' (' + method + ')')
     plt.legend(loc='upper left')
-    plt.xticks(x_pos, label, fontproperties=font)
+    plt.xticks(x_pos, label, rotation=45, fontproperties=font)
     plt.title(title)
     plt.show()
 
@@ -839,7 +840,7 @@ def bar04(df, parameter_x, parameter_y, method, title):  # single category
     plt.xlabel(parameter_x)
     plt.ylabel(parameter_y + ' (' + method + ')')
     plt.legend(loc='upper left')
-    plt.xticks(x_pos, label, fontproperties=font)
+    plt.xticks(x_pos, label, rotation=45, fontproperties=font)
     plt.title(title)
     plt.show()
 
@@ -1031,7 +1032,7 @@ def map01(df, province, parameter_x, method):
         data=dfnew,
         columns=['province', 'value'],
         key_on='feature.properties.name',
-        fill_color='YlOrBr',
+        fill_color='Greens',
         nan_fill_color='lightgrey',
         fill_opacity=0.8,
         nan_fill_opacity=0.1,
@@ -1305,10 +1306,155 @@ def scatter05box(df, parameter_x, parameter_y, title):
 
 
 
+
+
 # >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> [to be validated. update:12/28]
+# scatter plot + hist plot + linear regression
+def scatter06histlr(df, parameter_x, parameter_y, bins, title):
+    import matplotlib.pyplot as plt
+    from scipy.stats import linregress
+    import numpy as np
+
+    df = df
+    x = df[parameter_x]
+    y = df[parameter_y]
+    lr = linregress(x, y)
+    title = title
+
+    lr_label = '[L.R] y = ' + str("{0:.3g}".format(lr[0])) + ' *x + ' + str("{0:.3g}".format(lr[1])) + ' (r=' + str(
+        "{0:.3g}".format(lr[2])) + '|p=' + str("{0:.3g}".format(lr[3])) + '|σ=' + str("{0:.3g}".format(lr[4])) + ')'
+
+    # definitions for the axes
+    left, width = 0.1, 0.65
+    bottom, height = 0.1, 0.65
+    spacing = 0.005
+
+    rect_scatter = [left, bottom, width, height]
+    rect_histx = [left, bottom + height + spacing, width, 0.2]
+    rect_histy = [left + width + spacing, bottom, 0.2, height]
+
+    # start with a rectangular Figure
+    plt.figure(figsize=(8, 8))
+
+    # scatter plot
+    ax_scatter = plt.axes(rect_scatter)
+    plt.grid(linestyle='--', lw=0.5)
+    ax_scatter.tick_params(direction='in', top=True, right=True)
+    plt.xlabel(parameter_x)
+    plt.ylabel(parameter_y)
+
+    # top hist
+    ax_histx = plt.axes(rect_histx)
+    plt.grid(linestyle='--', lw=0.5, axis='y')
+    ax_histx.tick_params(direction='in', labelbottom=False)
+    plt.xlabel(parameter_x)
+    plt.ylabel('Counts')
+    plt.title(title)
+
+    # right hist
+    ax_histy = plt.axes(rect_histy)
+    plt.grid(linestyle='--', lw=0.5, axis='x')
+    ax_histy.tick_params(direction='in', labelleft=False)
+    plt.xlabel('Counts')
+    plt.ylabel(parameter_y)
+
+    # the scatter plot:
+    ax_scatter.scatter(x, y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='darkorange')
+
+    coef = np.polyfit(x, y, 1)
+    poly1d_fn = np.poly1d(coef)
+    ax_scatter.plot(x, poly1d_fn(x), '-', lw=1, color='blue', label=lr_label)
+    ax_scatter.legend(loc='lower left')
+
+    bins = bins
+
+    # now determine nice limits by hand:
+    ax_histx.hist(x, bins=bins, edgecolor='black', facecolor='darkorange', alpha=0.7)
+    ax_histy.hist(y, bins=bins, orientation='horizontal', edgecolor='black', facecolor='darkorange', alpha=0.7)
+
+    ax_histx.set_xlim(ax_scatter.get_xlim())
+    #     ax_histy.set_ylim(ax_scatter.get_ylim())
+
+    plt.show()
 
 
+# scatter plot + boxplot + linear regression
+def scatter07boxlr(df, parameter_x, parameter_y, title):
+    import matplotlib.pyplot as plt
+    import numpy as np
+    from scipy.stats import linregress
 
+    df = df
+    x = df[parameter_x]
+    y = df[parameter_y]
+
+    lr = linregress(x, y)
+    title = title
+
+    lr_label = '[L.R] y = ' + str("{0:.3g}".format(lr[0])) + ' *x + ' + str("{0:.3g}".format(lr[1])) + ' (r=' + str(
+        "{0:.3g}".format(lr[2])) + '|p=' + str("{0:.3g}".format(lr[3])) + '|σ=' + str("{0:.3g}".format(lr[4])) + ')'
+    #     print(lr_label)
+
+    # definitions for the axes
+    left, width = 0.1, 0.65
+    bottom, height = 0.1, 0.65
+    spacing = 0.005
+
+    rect_scatter = [left, bottom, width, height]
+    rect_histx = [left, bottom + height + spacing, width, 0.08]
+    rect_histy = [left + width + spacing, bottom, 0.08, height]
+
+    # start with a rectangular Figure
+    plt.figure(figsize=(8, 8))
+
+    # scatter plot
+    ax_scatter = plt.axes(rect_scatter)
+    plt.grid(linestyle='--', lw=0.5)
+    ax_scatter.tick_params(direction='in', top=True, right=True)
+    plt.xlabel(parameter_x)
+    plt.ylabel(parameter_y)
+
+    # top hist
+    ax_histx = plt.axes(rect_histx)
+    plt.grid(linestyle='--', lw=0.5, axis='y')
+    ax_histx.tick_params(direction='in', labelbottom=False)
+    plt.xlabel(parameter_x)
+    #     plt.ylabel('Counts')
+    plt.title(title)
+
+    # right hist
+    ax_histy = plt.axes(rect_histy)
+    plt.grid(linestyle='--', lw=0.5, axis='x')
+    ax_histy.tick_params(direction='in', labelleft=False)
+    #     plt.xlabel('Counts')
+    plt.ylabel(parameter_y)
+    # the scatter plot:
+    ax_scatter.scatter(x, y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='darkorange')
+
+    #     print(ploy1d_fn(x))
+    coef = np.polyfit(x, y, 1)
+    poly1d_fn = np.poly1d(coef)
+    ax_scatter.plot(x, poly1d_fn(x), '-', lw=1, color='blue', label=lr_label)
+    ax_scatter.legend(loc='lower left')
+
+    # now determine nice limits by hand:
+    #     ax_histx.hist(x, bins=bins,edgecolor='black',facecolor='darkorange',alpha=0.7)
+    #     ax_histy.hist(y, bins=bins, orientation='horizontal',edgecolor='black',facecolor='darkorange',alpha=0.7)
+
+    red_square = dict(markerfacecolor='lightgrey', marker='o', alpha=0.5, lw=1)
+
+    x = ax_histx.boxplot(x, flierprops=red_square, vert=False, widths=0.5, patch_artist=True)
+    for patch in x['boxes']:
+        patch.set_facecolor('orange')
+
+    y = ax_histy.boxplot(y, flierprops=red_square, widths=0.5, patch_artist=True)
+    for patch in y['boxes']:
+        patch.set_facecolor('orange')
+
+    #     ax_histx.set_xlim(ax_scatter.get_xlim())
+    #     ax_histy.set_ylim(ax_scatter.get_ylim())
+
+    plt.show()
 
 
 
